//-----------------------------------------------------------------------------
//
//						XMM Science Analysis System (SAS)
//					   (c) 1997-2010 European Space Agency
//
//-----------------------------------------------------------------------------
//
//	CalViewables.h	--- the things that can be viewed
//
//	DESCRIPTION:
//		Classes which implements the view() methods of CalViewable
//		objects. See also: CalView.h
//
//	CHANGE HISTORY:
//		UL, 1999-03-24: 1.0.0: - skeleton
//		UL, 1999-??-??: 1.1.0: - first ok version
//		UL, 1999-08-12: 1.2.0: - added CalViewable::isApplicable()
//		UL, 1999-09-15: 1.3.0: - added viewable: AbsCoef
//		UL, 1999-09-17: 1.4.0: - added viewable: CrossPsfPlot
//		UL, 2000-02-01: 1.5.0: - added viewable: FilterTransmission
//		UL, 2000-02-17: 1.6.0: - added viewable: QuantumEfficiency
//		UL, 2000-03-10: 1.7.0: - added viewable: EffectiveArea
//		UL, 2000-03-24: 1.8.0: - added viewable: Vignetting
//		UL, 2000-04-27: 1.9.0: - added viewable: GratingProperties
//							   - added Ae subviewables:
//									Ae = Ae(E, theta=const, phi=const)
//									Ae = Ae(E=const, theta, phi=const)
//									Ae = Ae(E=const, theta=const, phi)
//		UL, 2000-09-05: 1.10.0:- added Psf Viewable
//		UL, 2000-09-18: 1.11.0:- added encirced-energy Viewable
//		UL, 2000-11-22: 1.12.0:- added DispersedPsfPlot Viewable
//		UL, 2001-05-01: 1.13.0:- added DefocusLengthPlot
//
//	AUTHOR:
//		U. Lammers, Wed Mar 24 14:29:44 MET 1999
//
//	REFERENCES:
//		[1]: Software Specification Document for the XMM Science Analysis
//			 Subsystem, M. Dahlem et. al., 
//
//	$Header: /data/xmm12/cvsadmin/soc_repos/sas/packages/cal/calview/CalViewables.h,v 1.16 2001/05/01 12:41:21 ulammers Exp $
//

#ifndef CALVIEWABLE_H
#define CALVIEWABLE_H

#include "CalView.h"
#include "Tracer.h"

#define VIEWABLE_BASE(Name)												\
class Name : public CalViewable {										\
public:																	\
	Name(const CalViewServer *server = 0);								\
	CalViewProductDescriptor view(const string &setName = "") const;	\
	~Name() { PROC ENDPROC }

#define VIEWABLE(Name)		VIEWABLE_BASE(Name) }

#define VIEWABLE_APP(Name)	VIEWABLE_BASE(Name)	\
	bool isApplicable(Xmm::Instrument instrument) const; }


VIEWABLE(AbsCoefPlot);				//	absorption coefficients
VIEWABLE(BadPixPlot);				//	bad pixel map
VIEWABLE(CcdPlot);					//	show position of CCDs in camera coords.
VIEWABLE(HKParmIntPlot);			//	HK validity ranges

VIEWABLE_APP(CrossPsfPlot);			//	cross PSF of RGS
VIEWABLE_APP(DispersedPsfPlot);		//	cross PSF for RGS
VIEWABLE_APP(EffectiveAreaPlot);	//	effective area
VIEWABLE_APP(EncircledEnergyPlot);	//	encircled energy
VIEWABLE_APP(VignettingPlot);		//	vignetting
VIEWABLE_APP(PsfPlot);				//	PSF
VIEWABLE_APP(GratingPropertiesPlot);//	grating properties
VIEWABLE_APP(DefocusLengthPlot);	//	defocus length
VIEWABLE_APP(FilterTransPlot);		//	filter transmissions
VIEWABLE_APP(QuantumEfficiencyPlot);//	quantum efficiencies
VIEWABLE_APP(RedistributionPlot);	//	show cuts through RMF

#endif

