//
//	test the cal rawx2mm calls
//
#include <fstream>
#include <iostream>
#include "CalServer.h"
#include "Xmm.h"
#include "GeometryDataServer.h"
#include "common.h"

int main(int argc, char **argv)
{
    if (argc == 2) {
        //  assume the argument is a LINCOORD replacement
        const CifEntry e(argv[1]);
        calServer.ccf()->registerNewCifEntry(e);
    }

	for (unsigned i = 0; i < xmmPayLoad.instruments().size(); ++i) {

		XmmInstrument *ins = xmmPayLoad.instruments()[i];
		if (ins->id() == Xmm::ERM)
			continue;

		//	open xmgr parameter file
		strstream s;
		s<<"ccds_"<<lowercase(Xmm::toString(ins->id()))<<".dat"<<ends;
		ofstream set(s.str());
		set<<"@title \"Positions of CCDs in "<<Xmm::toString(ins->id())
		   <<"\"\n";
		set<<"@xaxis label \"x [mm]\"\n";
		set<<"@yaxis label \"y [mm]\"\n";
		set<<"@string loctype world\n";
		set<<"@string just 2\n";

		cerr<<"Instrument: "<<Xmm::toString(ins->id())<<endl;
		calServer.setInstrument(ins->id());
		calServer.state()->randomize()->set(false);
		if (calServer.state()->node()->isActive())
			calServer.state()->node()->set(Xmm::PRIMARY_NODE);
		if (calServer.state()->ocb()->isActive())
			calServer.state()->ocb()->set(1);

		for (unsigned ccd = 1; ccd<=ins->nCcds(); ++ccd) {
			calServer.state()->ccd()->set(ccd);

			short minX = ins->minX(),
				  maxX = ins->maxX(),
				  minY = ins->minY(),
				  maxY = ins->maxY();
			short rawX[] = { minX, maxX, maxX, minX, minX },
				  rawY[] = { minY, minY, maxY, maxY, minY };
			CalInt16Vector rawXVec(rawX, NUM_OF(rawX)),
						   rawYVec(rawY, NUM_OF(rawY));
			float xmm[NUM_OF(rawX)], ymm[NUM_OF(rawX)], zmm[NUM_OF(rawX)];

			GeometryDataServer *cdf = calServer.getAtom(cdf);
			XmmPixelList pl = cdf->chipCoordList(rawXVec, rawYVec);
			XmmCartesianPointList cpl = pl.toCamCoord1(xmm, ymm, zmm);
			cpl.toCamCoord2().toSacCoord();

			if (ins->id() == Xmm::RGS1 || ins->id() == Xmm::RGS2) {
				XmmRowlandPointList rpl = cpl.toRowCoord();
				CalReal32Vector d = cpl.rowlandDefocusLength();

				for (unsigned n=0; n<rpl.beta().size(); ++n)
					cout<<ccd<<" X/Y: "<<rawX[n]<<"/"<<rawY[n]
						<<" beta/chi="
						<<rpl.beta()[n]*RAD_TO_DEG<<"/"
						<<rpl.chi()[n]*RAD_TO_DEG
						<<" defocus length [mm]: "<<d[n]<<endl;
			}

			float cenx = .25 * (zmm[0]+zmm[1]+zmm[2]+zmm[3]),
				  ceny = .25 * (ymm[0]+ymm[1]+ymm[2]+ymm[3]);
			//
			//	write CCD number
			//
			set<<"@with string\n";
			set<<"@string on\n";
			set<<"@string char size 2\n";
			set<<"@string "<<cenx<<", "<<ceny<<endl;
			set<<"@string def \""<<ccd<<"\"\n";

			//
			//	coordinate systems: x
			//
			set<<"@with line\n";
			set<<"@line on\n";
			set<<"@line loctype world\n";
			set<<"@line "<<zmm[0]<<", "<<ymm[0]<<", "
						 <<zmm[1]<<", "<<ymm[1]<<endl;
			set<<"@line arrow 2\n";
			set<<"@line arrow size 1.5\n";
			set<<"@line def\n";
			set<<"@with string\n";
			set<<"@string on\n";
			set<<"@string char size 1\n";
			set<<"@string "<<zmm[1]<<", "<<ymm[1]<<endl;
			set<<"@string def \"X\"\n";

			//
			//	coordinate systems: y
			//
			set<<"@with line\n";
			set<<"@line on\n";
			set<<"@line loctype world\n";
			set<<"@line "<<zmm[0]<<", "<<ymm[0]<<", "
						 <<zmm[3]<<", "<<ymm[3]<<endl;
			set<<"@line arrow 2\n";
			set<<"@line arrow size 1.5\n";
			set<<"@line def\n";
			set<<"@with string\n";
			set<<"@string on\n";
			set<<"@string char size 1\n";
			set<<"@string "<<zmm[3]<<", "<<ymm[3]<<endl;
			set<<"@string def \"Y\"\n";

			//
			//	write data
			//
			set<<"@target s"<<ccd-1<<endl;
			set<<"@s"<<ccd-1<<" color 1\n";
			set<<"@type xy\n";

			for (unsigned n=0; n<NUM_OF(rawX); ++n)
				set<<zmm[n]<<" "<<ymm[n]<<endl;

			set<<"&\n";

		}
		set.close();
	}

	return 0;
}
