//
//	test the cal rawx2mm calls
//
#include <fstream>
#include <iostream>
#include <cmath>
#include "CalServer.h"
#include "Xmm.h"
#include "GeometryDataServer.h"
#include "common.h"

int main()
{
	for (unsigned i = 0; i < xmmPayLoad.instruments().size(); ++i) {

		XmmInstrument *ins = xmmPayLoad.instruments()[i];
		if (ins->id() == Xmm::ERM || ins->id() == Xmm::RGS1 ||
			ins->id() == Xmm::RGS2 || ins->id() == Xmm::OM)
			continue;

		cout<<"Instrument: "<<Xmm::toString(ins->id())<<endl;
		calServer.setInstrument(ins->id());
		calServer.state()->randomize()->set(false);
		if (calServer.state()->node()->isActive())
			calServer.state()->node()->set(Xmm::PRIMARY_NODE);
		if (calServer.state()->ocb()->isActive())
			calServer.state()->ocb()->set(1);

		for (unsigned ccd = 1; ccd<=ins->nCcds(); ++ccd) {
			calServer.state()->ccd()->set(ccd);
			cout<<"--- CCD: "<<ccd<<endl;

			short minX = ins->minX(),
				  maxX = ins->maxX(),
				  minY = ins->minY(),
				  maxY = ins->maxY();
			short rawX[] = { minX, maxX, maxX, minX, minX },
				  rawY[] = { minY, minY, maxY, maxY, minY };
			CalInt16Vector rawXVec(rawX, NUM_OF(rawX)),
						   rawYVec(rawY, NUM_OF(rawY));
			float xmm[NUM_OF(rawX)], ymm[NUM_OF(rawX)], zmm[NUM_OF(rawX)];
			float theta[NUM_OF(rawX)], phi[NUM_OF(rawY)];

			GeometryDataServer *cdf = calServer.getAtom(cdf);
			XmmPixelList pl = cdf->chipCoordList(rawXVec, rawYVec);
			XmmCartesianPointList cpl =
					pl.toCamCoord1(xmm, ymm, zmm).toCamCoord2();

			XmmTelescopePointList tpl = cpl.toTelCoord(theta, phi);

			CalReal32Vector convX(0, NUM_OF(rawX)), convY(0, NUM_OF(rawX)),
						    convZ(0, NUM_OF(rawX));
			XmmCartesianPointList convpl =
					tpl.toCamCoord2(convX, convY, convZ);

			CalInt16Vector cx(0, NUM_OF(rawX)), cy(0, NUM_OF(rawY));
			XmmPixelList convpxl = convpl.toCamCoord1().toChipCoord(cx, cy);

			for (unsigned n=0; n<NUM_OF(rawX); ++n) {
				cout<<rawX[n]<<"/"<<rawY[n]
					<<" = "<<theta[n]<<"/"<<phi[n]*RAD_TO_DEG<<" = "
					<<cx[n]<<"/"<<cy[n]<<endl;
				assert(rawX[n] == cx[n] && rawY[n] == cy[n]);
			}
			
		}
	}
	return 0;
}
