//
//	test the CcdQuantumDataServer class
//
#include <iostream>
#include <cassert>
#include "CalServer.h"
#include "Ccf.h"
#include "CcdQuantumDataServer.h"

namespace {
bool isClose(double a, double b)
{
    return fabs(a)/fabs(b)-1. < 1.e-5;
}
}

int main()
{
	//	setup the the Cal server for MOS
	Xmm::Instrument instrument = Xmm::EMOS1;
	calServer.setInstrument(instrument);
	calServer.state()->ccd()->set(1);
	calServer.state()->node()->set(EMOScam::PRIMARY_NODE);
	calServer.state()->filter()->set(Xmm::THIN1);

	//	get the QE DataServer
	CcdQuantumDataServer *qds = calServer.getAtom(qds);

	double e[] = { 100., 1000., 3000., 5000., 9000. },
	f1v[] = {-0.0690878, 0.568432, 0.686166, 0.497647, 0.143326 }, 
	f2v[] = { -0.0715211, 0.570339, 0.686137, 0.49693, 0.14316 };

	const CcdQuantumEfficiencyCurve &ef = qds->at(300, 300).pattern(0);
	for (unsigned n=0; n<NUM_OF(e); ++n) {
		double ft = ef.efficiency(e[n]),
			   fi = ef.efficiency(e[n], e[n]+10.);
		cout<<"QE at E="<<e[n]<<"="<<ft
			<<" - integrated over 10 eV: "<<fi<<endl;
		assert(isClose(ft, f1v[n]) && isClose(fi, f2v[n]));
	}
	calServer.releaseAtom(qds);

	//	setup the the Cal server for RGS
	instrument = Xmm::RGS1;
	calServer.setInstrument(instrument);
	calServer.state()->ccd()->set(5);

	//	get the QE DataServer
	qds = calServer.getAtom(qds);

	double e2[] = { 100., 1000., 3000., 5000., 9000. },
		   f3v[] = { 0.0500342, 0.943739, 0.980199, 0.790405, 0.250872 },
		   f4v[] = { 0.0509488, 0.944441, 0.980239, 0.789496, 0.250521 };

	const CcdQuantumEfficiencyCurve &ef2 = qds->grade(1);
	for (unsigned n=0; n<NUM_OF(e2); ++n) {
		double ft = ef2.efficiency(e2[n]),
			   fi = ef2.efficiency(e2[n], e2[n]+10.);
		cout<<"QE at E="<<e2[n]<<"="<<ft
			<<" - integrated over 10 eV: "<<fi<<endl;
		assert(isClose(ft, f3v[n]) && isClose(fi, f4v[n]));
	}

	return 0;
}
