//-----------------------------------------------------------------------------
//
//						XMM Science Analysis System (SAS)
//					   (c) 1997-2010 European Space Agency
//
//-----------------------------------------------------------------------------
//
//	BadPixelDataServer.h --- classes related to bad pixels
//
//	DESCRIPTION:
//
//	CHANGE HISTORY:
//		UL, 2001-02-19: 1.0.0: - first version from CalAtomsCommon.h
//		UL, 2001-03-22: 1.1.0: - moved BadPixel/BadPixelMap classes to
//								 caloalutils
//
//	AUTHOR:
//		U. Lammers, Mon Oct  6 15:32:57 WET DST 1997
//
//	REFERENCES:
//		[1]: Software Specification Document for the XMM Science Analysis
//			 Subsystem, M. Dahlem et. al., 
//
//	$Header: /data/xmm12/cvsadmin/soc_repos/sas/packages/cal/src/BadPixelDataServer.h,v 1.4 2001/03/22 18:48:37 ulammers Exp $
//

#ifndef BAD_PIXEL_DATA_SERVER_H
#define BAD_PIXEL_DATA_SERVER_H

#include "Ccf.h"
#include "CalAtomsBase.h"
#include "BadPixelMap.h"

#include <vector>

class BadPixelDataServer : public CalAtom,
	public virtual CalChild<CalAtomCollection<BadPixelDataServer> > {
public:
	BadPixelDataServer(CalAtomCollection<BadPixelDataServer> *parent = 0);
		//
		//	PURPOSE:
		//		Instantiate bad pixel data server object
		//


	static unsigned id() { PROC RETURN Ccf::BadPix; }
		//
		//	PURPOSE:
		//		Return ID of CalAtom
		//


	BadPixelMap map() const;
		//
		//	PURPOSE:
		//		Obtain the list of bad pixels applicable for current
		//		instrument mode.
		//


	~BadPixelDataServer();
		//
		//	PURPOSE:
		//		Destroy object constructed with above constructor(s).
		//

private:
	friend class CalAtomCollection<BadPixelDataServer>;

	typedef struct {
		PixelLocation			loc;
		BadPixel				bp;
		vector<short>			modena;
	} LocBadPixModeType;
	vector<LocBadPixModeType>	_badPixels;
};

#endif

