//-----------------------------------------------------------------------------
//
//						XMM Science Analysis System (SAS)
//					   (c) 1997-2010 European Space Agency
//
//-----------------------------------------------------------------------------
//
//	CalAtomServer.h --- give access to calibration atoms
//
//	DESCRIPTION:
//
//	CHANGE HISTORY:
//		UL, 24/04/98: 1.0.0: - skeleton
//
//	AUTHOR:
//		UL, Fri Apr 24 09:00:59 MET DST 1998
//
//	REFERENCES:
//		[1]: Software Specification Document for the XMM Science Analysis
//			 Subsystem, M. Dahlem et. al., 
//
//	$Header: /data/xmm12/cvsadmin/soc_repos/sas/packages/cal/src/CalAtomServer.h,v 1.30 2001/02/19 15:39:49 ulammers Exp $
//

#ifndef CAL_ATOM_SERVER_H
#define CAL_ATOM_SERVER_H

#include "Tracer.h"
#include "common.h"
#include "Xmm.h"
#include "CalAtomsBase.h"
#include "Ccf.h"
#include "CalChild.h"

#include "AbsorptionDataServer.h"
#include "BadPixelDataServer.h"
#include "Boresight.h"
#include "GeometryDataServer.h"
#include "CalAtomsMisc.h"
#include "CcdQuantumDataServer.h"
#include "CrossPsf.h"
#include "CtiCorrector.h"
#include "DarkFrameDataServer.h"
#include "EffectiveAreaDataServer.h"
#include "EncircledEnergyDataServer.h"
#include "EnergyCorrector.h"
#include "FilterDataServer.h"
#include "GratingDataServer.h"
#include "HKvalRangesServer.h"
#include "MiscDataServer.h"
#include "OmColorTransformator.h"
#include "OmFlatFieldServer.h"
#include "OmFluxConverter.h"
#include "OmGeoLinearizer.h"
#include "OmPsfDataServer.h"
#include "PatternLibServer.h"
#include "PsfDataServer.h"
#include "Redistribution.h"
#include "TimeCorrector.h"

#include <vector>

class CalServer;

class CalAtomCollectionServer : public virtual CalChild<CalServer> {
public:
	//	construct from CCF instance - default constructor does not make sense
	CalAtomCollectionServer(const CalServer *server);

	void restart();
		//
		//	PURPOSE:
		//		Flush all memory-buffered atoms; this effectively triggers
		//		re-initializtion of atoms by re-reading CCF constituents
		//

	template <class AtomType>
	CalAtomCollection<AtomType> *getCollection(const XmmInstrument *,
											   AtomType *) const;

	template <class AtomType>
	void releaseCollection(const XmmInstrument *, AtomType *) const;

	~CalAtomCollectionServer();

private:
	unsigned index(const Xmm::Instrument instrument,
				   unsigned) const;
	CalAtomCollectionBase *getEntry(Xmm::Instrument, unsigned) const;
	void setEntry(Xmm::Instrument, unsigned, CalAtomCollectionBase *) const;
	void releaseColl(Xmm::Instrument, unsigned) const;
	vector<CalAtomCollectionBase *> *_atomsCollection;
};

#endif

