//-----------------------------------------------------------------------------
//
//						XMM Science Analysis System (SAS)
//					   (c) 1997-2010 European Space Agency
//
//-----------------------------------------------------------------------------
//
//	CalDataVector.h	--- classes representing a data vector
//
//	DESCRIPTION:
//		Some simple wrapper classes around pointers to integer/real vectors
//
//	CHANGE HISTORY:
//		UL, 1999-05-31: 1.0.0: - first version
//		UL, 1999-11-24: 1.1.0: - added CalInt32Vector
//
//	AUTHOR:
//		U. Lammers, Mon May 31 16:45:25 MET DST 1999
//
//	REFERENCES:
//		[1]: Software Specification Document for the XMM Science Analysis
//			 Subsystem, M. Dahlem et. al., 
//
//	$Header: /data/xmm12/cvsadmin/soc_repos/sas/packages/cal/src/CalDataVector.h,v 1.9 2001/02/21 15:25:56 ulammers Exp $
//

#ifndef CAL_DATAVECTOR_H
#define CAL_DATAVECTOR_H

#include "Tracer.h"
#include "Dal.h"
#include "CalChild.h"

class CalServer;

template<class ElementType>
class CalVector {
public:
	CalVector(ElementType *ptr = 0, unsigned n = 0);
	//	copy constructor (performs shallow copy)
	CalVector(const CalVector<ElementType> &);
	inline ElementType &operator[](unsigned n) const { return _ptr[n]; }
	inline ElementType *ptr() const { return _ptr; }
	inline void free() { delete [] _ptr; _ptr = 0; }
	inline unsigned size() const { return _n; }

	//	do a deep copy
	void clone(const CalVector<ElementType> &source);
	CalVector<ElementType> clone() const { CalVector<ElementType> v;
											v.clone(*this); return v; }

	virtual ~CalVector() {}

protected:
	ElementType *_ptr;
	unsigned    _n;
	bool        _memAllocated;
}; // CalVector()


template<class RealType> class CalRealVector;

template<class IntType>
class CalIntVector : public CalVector<IntType>,
					 public virtual CalChild<CalServer> {
public:
	CalIntVector(IntType *ptr = 0, unsigned n = 0,
				 const CalServer *server = 0);

	CalRealVector<real32> &toReal(CalRealVector<real32> &rv) const;

	~CalIntVector() {}
}; // CalIntVector


template<class RealType>
class CalRealVector : public CalVector<RealType>,
					  public virtual CalChild<CalServer> {
public:
	CalRealVector(RealType *ptr = 0, unsigned n = 0,
				  const CalServer *server = 0);

	CalRealVector<RealType> clone() const { CalRealVector<RealType> v;
											CalVector<RealType> &vc = v;
											vc.clone(*this); return v; }

	template<class ToIntType>
	CalIntVector<ToIntType> &toInt(CalIntVector<ToIntType> &iv) const;

	~CalRealVector() {}
}; // CalRealVector

typedef CalIntVector<int8>				CalInt8Vector;
typedef CalIntVector<int16>             CalInt16Vector;
typedef CalIntVector<int32>             CalInt32Vector;
typedef CalRealVector<real32>           CalReal32Vector;
typedef CalRealVector<real64>			CalReal64Vector;

#endif
