//-----------------------------------------------------------------------------
//
//						XMM Science Analysis System (SAS)
//					   (c) 1997-2010 European Space Agency
//
//-----------------------------------------------------------------------------
//
//	DarkFrameDataServer --- the DarkFrameDataServer class
//
//	DESCRIPTION:
//		A class 'DarkFrameDataServer' is defined which provides access to
//		darkframe data in the CCF
//
//	CHANGE HISTORY:
//		UL, 2000-07-28: 1.0.0: - first version
//		UL, 2001-04-23: 1.1.0: - changed type of darkFrame map from int16
//								 to real32
//
//	AUTHOR:
//		U. Lammers (UL), Fri Jul 28 13:50:02 MET DST 2000
//
//	REFERENCES:
//		[1]: Software Specification Document for the XMM Science Analysis
//			 Subsystem, M. Dahlem et. al.,
//		[2]: Calibration Data Access Handbool, C. Erd et al.
//
//	$Header: /data/xmm12/cvsadmin/soc_repos/sas/packages/cal/src/DarkFrameDataServer.h,v 1.3 2001/04/23 16:17:54 ulammers Exp $
//

#ifndef DARKFRAME_DATA_SERVER_H
#define DARKFRAME_DATA_SERVER_H

#include "Tracer.h"
#include "Ccf.h"
#include "CalChild.h"
#include "CalAtomsBase.h"
#include "Image.h"

//
//	---( DarkFrameDataServer )----------------------------------------
//
class DarkFrameDataServer : public CalAtom,
	 public virtual CalChild<CalAtomCollection<DarkFrameDataServer> > {
public:
	DarkFrameDataServer(unsigned modeOffset,
						const Proxy<SImage> &darkFrameImage,
						const Table *table,
						CalAtomCollection<DarkFrameDataServer> *parent = 0);
		//
		//	PURPOSE:
		//		Instantiate DarkFrameDataServer object from table to
		//		CCF::DARKFRAME table.
		//


	virtual void dump(ostream &) { PROC ENDPROC }
		//
		//	PURPOSE:
		//		Dump data members to an ostream
		//
		//	ARGUMENTS:
		//		In:
		//			os:	stream to print on
		//


	static unsigned id() { PROC RETURN Ccf::DarkFrame; }
		//
		//	PURPOSE:
		//		Return ID of CalAtom
		//


	const vector<int16> &rowOffsetVector() const;
	const vector<int16> &columnOffsetVector() const;
		//
		//	PURPOSE:
		//		Return reference to row/column offset vectors;
		//		to be indexed in PIXCOORD0 system (!)
		//


	const SImage &darkFrameMap() const;
		//
		//	PURPOSE:
		//		Return reference to dark frame map (in PIXCOORD1 frame)
		//


	virtual ~DarkFrameDataServer();
		//
		//	PURPOSE:
		//		Destroy object constructed with above constructor(s).
		//

private:
	//	data members
	unsigned		_modeOffset;
	Proxy<SImage> _darkFrameImage;
	vector<int16>	*_offX,
					*_offY;
}; // DarkFrameDataServer

#endif
