//-----------------------------------------------------------------------------
//
//						XMM Science Analysis System (SAS)
//					   (c) 1997-2010 European Space Agency
//
//-----------------------------------------------------------------------------
//
//	MiscDataServer.h --- class to retrieve miscellaneous CCF data
//
//	DESCRIPTION:
//		A class MiscDataServer is declared instance of which are to be
//		used to retrive single-valued data from CCF::MiscData.
//		The query is keyword based.
//
//	CHANGE HISTORY:
//		UL, 1997-10-25: 1.0.0: - skeleton
//		UL, 1999-09-08: 2.0.0: - overhauled from CalAtomsXmm
//
//	AUTHOR:
//		U. Lammers, Mon Oct  6 15:32:57 WET DST 1997
//
//	REFERENCES:
//		[1]: Software Specification Document for the XMM Science Analysis
//			 Subsystem, M. Dahlem et. al., 
//
//	$Header: /data/xmm12/cvsadmin/soc_repos/sas/packages/cal/src/MiscDataServer.h,v 1.5 2000/05/05 14:55:44 ulammers Exp $
//

#ifndef MISC_DATA_SERVER_H
#define MISC_DATA_SERVER_H

#include <map>

#include "Ccf.h"
#include "CalChild.h"
#include "CalAtomsBase.h"

class MiscDataValue;	//	forward declaration
typedef map<string, const MiscDataValue *, less<string> >	MiscDataMapType;

//
//	---( MiscDataServer )---------------------------------------------------
//
class MiscDataServer : public CalAtom,
				   public virtual CalChild<CalAtomCollection<MiscDataServer> >,
				   public MiscDataMapType {
public:
	MiscDataServer(const string &tableName,
				   CalAtomCollection<MiscDataServer> *parent = 0)
		: CalChild<CalAtomCollection<MiscDataServer> >(parent),
		  _tableName(tableName) { PROC ENDPROC }
		//
		//	PURPOSE:
		//		Instantiate a MiscDataServer object
		//


	static unsigned id() { PROC RETURN Ccf::MiscData; }
		//
		//	PURPOSE:
		//		Return a uniue ID which for mapping Cal atoms onto
		//		CCF constituents
		//


	virtual void dump(ostream &os);
		//
		//	PURPOSE:
		//		Dump data members to an ostream
		//
		//	ARGUMENTS:
		//		In:
		//			os:	stream to print on
		//


	virtual bool has(const string &keyword) const;
		//
		//	PURPOSE:
		//		Check whether a data item with the specified keyword
		//		exists
		//


	virtual double get(const string &keyword) const;
		//
		//	PURPOSE:
		//		Retrieve the value of the miscellaneous data item
		//		with the keyword <keyword>; Please note: attempting
		//		to retrieve a non-existing data item will result in
		//		an error condition.


	virtual ~MiscDataServer();
		//
		//	PURPOSE:
		//		Destrory MiscDataServer object constructed with above
		//		constructor(s).
		//

private:
	double getValueFromOtherConstituent(const string &) const;
	string	_tableName;
};
	
#endif


